@@ -34,6 +34,10 @@ public class AlgorithmIdentifiers
     public static final String ECDSA_USING_P384_CURVE_AND_SHA384 = "ES384";
     public static final String ECDSA_USING_P521_CURVE_AND_SHA512 = "ES512";
 
+    public static final String ECDSA_USING_SECP256K1_CURVE_AND_SHA256 = "ES256K";
+
+    public static final String EDDSA = "EdDSA";
+
     public static final String RSA_PSS_USING_SHA256 = "PS256";
     public static final String RSA_PSS_USING_SHA384 = "PS384";
     public static final String RSA_PSS_USING_SHA512 = "PS512";
